from stream_django.conf import FEED_MANAGER_CLASS
from stream_django.conf import DJANGO_MAJOR_VERSION
from django.db.models.signals import class_prepared
from stream_django.utils import get_class_from_string


feed_manager_class = get_class_from_string(FEED_MANAGER_CLASS)
feed_manager = feed_manager_class()

class_prepared = class_prepared.connect(feed_manager.bind_model)


def disable_model_tracking(sender, **kwargs):
    feed_manager.disable_model_tracking()


if DJANGO_MAJOR_VERSION == 1.6:
    from django.db.models.signals import pre_syncdb
    pre_syncdb.connect(disable_model_tracking)
elif DJANGO_MAJOR_VERSION > 1.6:
    from django.db.models.signals import pre_migrate
    pre_migrate.connect(disable_model_tracking)



import django
from django import template
from django.template import Context, loader
from stream_django.exceptions import MissingDataException
import logging


logger = logging.getLogger(__name__)
register = template.Library()

LOG = 'warn'
IGNORE = 'ignore'
FAIL = 'fail'

missing_data_policies = [LOG, IGNORE, FAIL]


def handle_not_enriched_data(activity, policy):
    message = 'could not enrich field(s) %r for activity #%s' % (activity.not_enriched_data, activity.get('id'))
    if policy == IGNORE:
        pass
    elif policy == FAIL:
        raise MissingDataException(message)
    elif policy == LOG:
        logger.warn(message)
    else:
        raise TypeError('%s is not a valid missing_data_policy' % policy)


def render_activity(context, activity, template_prefix='', missing_data_policy=LOG):
    if hasattr(activity, 'enriched') and not activity.enriched:
        handle_not_enriched_data(activity, missing_data_policy)
        return ''

    if template_prefix != '':
        template_prefix = '%s_' % template_prefix

    if 'activities' in activity:
        template_name = "activity/aggregated/%s%s.html" % (template_prefix, activity['verb'])
    else:
        template_name = "activity/%s%s.html" % (template_prefix, activity['verb'])

    tmpl = loader.get_template(template_name)
    context['activity'] = activity

    if django.VERSION < (1, 11):
        context = Context(context)
    elif isinstance(context, Context):
        context = context.flatten()

    return tmpl.render(context)


register.simple_tag(takes_context=True)(render_activity)



def get_class_from_string(path, default=None):
    """
    Return the class specified by the string.

    IE: django.contrib.auth.models.User
    Will return the user class or cause an ImportError
    """
    try:
        from importlib import import_module
    except ImportError:
        from django.utils.importlib import import_module
    i = path.rfind('.')
    module, attr = path[:i], path[i + 1:]
    mod = import_module(module)
    try:
        return getattr(mod, attr)
    except AttributeError:
        if default:
            return default
        else:
            raise ImportError(
                'Cannot import name {} (from {})'.format(attr, mod))



from django.apps import AppConfig
from django.core.exceptions import ImproperlyConfigured
from stream_django.conf import is_valid_config

class StreamDjangoConfig(AppConfig):
    name = 'stream_django'
    verbose_name = "Stream Django"

    def ready(self):
      if not is_valid_config():
        raise ImproperlyConfigured('Stream credentials are not set in your settings')



from stream_django import conf
import os
import stream
from stream_django.conf import DJANGO_MAJOR_VERSION
from django.core.exceptions import ImproperlyConfigured

def init_client(raise_config_error=False):
  if conf.API_KEY and conf.API_SECRET:
      return stream.connect(conf.API_KEY, conf.API_SECRET, location=conf.LOCATION, timeout=conf.TIMEOUT)
  elif os.environ.get('STREAM_URL') is not None:
      return stream.connect()
  elif raise_config_error:
      raise ImproperlyConfigured('Stream credentials are not set in your settings')

stream_client = init_client(raise_config_error=DJANGO_MAJOR_VERSION<1.7)




import os
import sys
from setuptools import setup


with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

if sys.version_info < (3, 0, 0):
    django = 'django>=1.5,<2.0'
else:
    django = 'django>=1.5,<2.1.0'

requirements = [
    django,
    'stream-python>=2.8.1',
    'pytz'
]

extras_require = {
    'test': ['httpretty', 'coverage'],
}

setup(
    name='stream-django',
    version='1.4.0',
    packages=['stream_django'],
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    tests_require=['stream_django[test]'],
    test_suite='runtests.runtests',
    license='BSD License',
    description='A Django app to build activity, news and notification feeds.',
    long_description=README,
    url='https://getstream.io/',
    author='Tommaso Barbugli',
    author_email='tbarbugli@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



import django
import os
from django.conf import settings

API_KEY = getattr(settings, 'STREAM_API_KEY', None)
API_SECRET = getattr(settings, 'STREAM_API_SECRET', None)
LOCATION = getattr(settings, 'STREAM_LOCATION', None)
TIMEOUT = getattr(settings, 'STREAM_TIMEOUT', 6.0)

FEED_MANAGER_CLASS = getattr(settings, 'STREAM_FEED_MANAGER_CLASS',
    'stream_django.managers.FeedManager'
)

USER_FEED = getattr(settings, 'STREAM_USER_FEED', 'user')
NEWS_FEEDS = getattr(settings, 'STREAM_NEWS_FEEDS',
    {'timeline':'timeline', 'timeline_aggregated':'timeline_aggregated'}
)
NOTIFICATION_FEED = getattr(settings, 'STREAM_PERSONAL_FEED', 'notification')

DISABLE_MODEL_TRACKING = getattr(settings, 'STREAM_DISABLE_MODEL_TRACKING', False)

DJANGO_MAJOR_VERSION = django.VERSION[0] + (0.1*django.VERSION[1])

def is_valid_config():
  return (API_KEY and API_SECRET) or os.environ.get('STREAM_URL')



import collections
from collections import defaultdict
import operator
import itertools

try:
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import get_model


def combine_dicts(a, b, op=operator.add):
    return dict(list(a.items()) + list(b.items()) +
        [(k, op(a[k], b[k])) for k in set(b) & set(a)])


DEFAULT_FIELDS = ('actor', 'object')


class EnrichedActivity(collections.MutableMapping):

    def __init__(self, activity_data):
        self.activity_data = activity_data
        self.not_enriched_data = {}

    def __getitem__(self, key):
        return self.activity_data[self.__keytransform__(key)]

    def __setitem__(self, key, value):
        self.activity_data[self.__keytransform__(key)] = value

    def __delitem__(self, key):
        del self.activity_data[self.__keytransform__(key)]

    def __iter__(self):
        return iter(self.activity_data)

    def __len__(self):
        return len(self.activity_data)

    def __keytransform__(self, key):
        return key

    def __repr__(self):
        return "EnrichedActivity(activity_data=%s, not_enriched_data=%s)" % (str(self.activity_data),
                                                                             str(self.not_enriched_data))

    def track_not_enriched_field(self, field, value):
        self.not_enriched_data[field] = value

    @property
    def enriched(self):
        return len(self.not_enriched_data.keys()) == 0


class Enrich(object):

    def __init__(self, fields=DEFAULT_FIELDS):
        self.fields = fields

    def enrich_aggregated_activities(self, activities):
        references = {}
        for activity in activities:
            activity['activities'] = self.wrap_activities(activity['activities'])
            references = combine_dicts(references, self._collect_references(activity['activities'], self.fields))
        objects = self._fetch_objects(references)
        for activity in activities:
            self._inject_objects(activity['activities'], objects, self.fields)
        return activities

    def enrich_activities(self, activities):
        activities = self.wrap_activities(activities)
        references = self._collect_references(activities, self.fields)
        objects = self._fetch_objects(references)
        self._inject_objects(activities, objects, self.fields)
        return activities

    def wrap_activities(self, activities):
        return [EnrichedActivity(a) for a in activities]

    def is_ref(self, activity, field):
        return len(activity[field].split(':')) == 2 if activity.get(field) else False

    def _collect_references(self, activities, fields):
        model_references = defaultdict(list)
        for activity, field in itertools.product(activities, fields):
            if not self.is_ref(activity, field):
                continue
            f_ct, f_id = activity[field].split(':')
            model_references[f_ct].append(f_id)
        return model_references

    def fetch_model_instances(self, modelClass, pks):
        '''
        returns a dict {id:modelInstance} with instances of model modelClass
        and pk in pks
        '''
        hook_function_name = 'fetch_%s_instances' % (modelClass._meta.object_name.lower(), )
        if hasattr(self, hook_function_name):
            return getattr(self, hook_function_name)(pks)
        qs = modelClass.objects
        if hasattr(modelClass, 'activity_related_models') and modelClass.activity_related_models() is not None:
            qs = qs.select_related(*modelClass.activity_related_models())
        return qs.in_bulk(pks)

    def _fetch_objects(self, references):
        objects = defaultdict(list)
        for content_type, ids in references.items():
            model = get_model(*content_type.split('.'))
            ids = set(ids)
            instances = self.fetch_model_instances(model, ids)
            objects[content_type] = instances
        return objects

    def _inject_objects(self, activities, objects, fields):
        for activity, field in itertools.product(activities, fields):
            if not self.is_ref(activity, field):
                continue
            f_ct, f_id = activity[field].split(':')
            model = get_model(*f_ct.split('.'))
            f_id = model._meta.pk.to_python(f_id)

            instance = objects[f_ct].get(f_id)
            if instance is None:
                activity.track_not_enriched_field(field, activity[field])
            else:
                activity[field] = instance
